{*************************************
 *                                   *
 *        Communication & Co...      *
 *                                   *
 *************************************}

Type ComSolo = Object(ComX)
         Function  Init( Adr : Word; Irq : Byte;
                         Speed : Word; Param : Byte;
                         LgIn, LgOut : Word ) : Boolean;
         Procedure Done;
         Procedure Clear;
         Procedure Write( texte : string );
         Procedure BlockWrite( Var p; Lg : Word );
         Procedure WaitOutEmpty;
       End;


Function  CalcCheckUpSpeed( Baud : Word ) : Byte; Assembler;
Asm
    MOV  AX, 640
    XOR  DX, DX
    DIV  [Baud]
    MOV  CL, 16
@ENCORE:
    SHL  AX, 1
    JC   @FIN
    LOOP @ENCORE
@FIN:
    MOV  AL, CL
End;


Function  ComSolo.Init;
Begin
 If (Irq=0) And Not(InstCheckUp(CalcCheckUpSpeed(Speed))) Then Init := FALSE
 Else
  If ComX.Init(Adr,Irq,Speed,Param,LgIn,LgOut) Then Init := TRUE
  Else
   Begin
    If Irq=0 Then KillCheckUp;
    Init := FALSE;
   End;
End;


Procedure  ComSolo.Done;
Begin
 ComX.Done;
 KillCheckUp;
End;


Procedure ComSolo.Clear;
Begin
 ReceiveClear;
 TransmitClear;
End;


Procedure ComSolo.Write( texte : string );
Var i : Word;
Begin
 i := 0;
 While i<Length(texte) Do Inc(i,Transmit(texte[i+1],Length(texte)-i));
End;


Procedure ComSolo.BlockWrite( Var p; Lg : Word );
Type Buffer = Array[0..10000] Of Byte;
Var i : Word;
Begin
 i := 0;
 While i<Lg Do Inc(i,Transmit(Buffer(p)[i],Lg-i));
End;


Procedure ComSolo.WaitOutEmpty;
Begin
 While TransmitCount>0 Do;
End;


Var Com : ComSolo;


Var ExInt1C  : Pointer;
    SaveExit : Pointer;
    C18      : Longint;

    ExInt1B  : Pointer;
    BreakDetect : Boolean;


{$IFOPT R+ } {$DEFINE RON } {$R-} {$ENDIF }
{$IFOPT S+ } {$DEFINE SON } {$S-} {$ENDIF }

Procedure NewInt1C; Assembler;
Asm
 PUSH AX
 PUSH DS

 MOV  AX, SEG @DATA
 MOV  DS, AX

 ADD  [WORD PTR C18], 1
 ADC  [WORD PTR C18+2], 0

 PUSHF
 CALL [ExInt1C]

 POP  DS
 POP  AX
 IRET
End;


Procedure NewInt1B; Assembler;
Asm
 PUSH AX
 PUSH DS

 MOV  AX, SEG @DATA
 MOV  DS, AX

 MOV  [BreakDetect], TRUE

 PUSHF
 CALL [ExInt1B]

 POP  DS
 POP  AX
 IRET
End;


{$IFDEF SON } {$S+ } {$UNDEF SON } {$ENDIF }
{$IFDEF RON } {$R+ } {$UNDEF RON } {$ENDIF }


Function GetC18 : Longint; Assembler;
Asm
    CLI
    MOV  AX, [WORD PTR C18]
    MOV  DX, [WORD PTR C18+2]
    STI
End;


Const MPRO1      = #27#$39;
      MPRO2      = #27#$3A;
      MPRO3      = #27#$3B;
      MOFF       = #$60;
      MON        = #$61;
      MSTART     = #$69;
      MSTOP      = #$6A;
      MROULEAU   = #$43;
      MEEcran    = #$50;   { module emeteur code }
      MEClavier  = #$51;
      MEModem    = #$52;
      MEPrise    = #$53;
      MREcran    = #$58;   { module recepteur code }
      MRClavier  = #$59;
      MRModem    = #$5A;
      MRPrise    = #$5B;



Const SPEED_NIL = $FF;

      Speed75   = 0;
      Speed300  = 1;
      Speed1200 = 2;
      Speed4800 = 3;
      Speed9600 = 4;

      LastSpeed : Byte = SPEED_NIL; { la vitesse de dpart du minitel! }

      MinSpeed  : Byte = Speed1200;
      MinSlow   : Boolean = FALSE;


Type SpeedRec = Record
       BD      : Word;
       CSpeed  : Char;
      End;

Const NbSpeed = 5;

      TabSpeed : Array[0..NbSpeed-1] Of SpeedRec = (
       ( BD:BAUD75;   CSpeed:Char(64+9) ),
       ( BD:BAUD300;  CSpeed:Char(64+18) ),
       ( BD:BAUD1200; CSpeed:Char(64+36) ),
       ( BD:BAUD4800; CSpeed:Char(64+54) ),
       ( BD:BAUD9600; CSpeed:Char(64+63) )
       );


Function WaitInStr( Texte : String; TiNoCode, TiMax : Word;
                    Var Res : String ) : Boolean;
Var iRead : Byte;
    TiDeb : Longint;
    TiNo  : Longint;
    Ti    : Longint;
    c     : Char;
Begin
 iRead := 0;
 TiDeb := GetC18;

 While iRead<Length(Texte) Do
  Begin
   TiNo := GetC18;
   Repeat
    Ti := GetC18;
    If ((Ti-TiNo)>=TiNoCode) Or ((Ti-TiDeb)>=TiMax) Then
     Begin
      WaitInStr := FALSE;
      Exit;
     End;
   Until Com.ReceiveChar(c) And (c<>#0);  { Ignore les 0 }

   Inc(iRead);
   If Not((Texte[iRead]=#0) Or (Texte[iRead]=c)) Then { Recommencer ! }
    If (Texte[1]=#0) Or (Texte[1]=c) Then iRead := 1 Else iRead := 0
   Else
    Begin
     Res[iRead] := c;
    End;
  End;

 Res[0] := Texte[0];
 WaitInStr := TRUE;
End;


Function _CalcC18Delay( Len : Word; Speed : Word; Slow : Boolean ) : Word;
Var x : Word;
Begin
 x := 11520 Div TabSpeed[Speed].BD;
 x := ((Len+2+2)*18) Div x;
 If Slow Then Inc(x,6) Else Inc(x,3);
 _CalcC18Delay := x;
End;

Function CalcC18Delay( Len : Word ) : Word;
Begin
 CalcC18Delay := _CalcC18Delay(Len,MinSpeed,MinSlow);
End;


Function MinDetect( WantSpeed : Byte; Slow, AutoSeek : Boolean ) : Boolean;
Const OrderRank : Array[0..NbSpeed-1] of byte =
        (Speed1200,Speed4800,Speed9600,Speed300,Speed75);
Var SpeedRank, SpeedCourant : Byte;
    i         : Byte;
    TabUse    : Array[0..NbSpeed-1] Of Boolean;
    x         : Word;
    Res       : String;
begin
 For i := 0 To NbSpeed-1 Do TabUse[i] := FALSE;
 SpeedRank := 0;
 SpeedCourant := WantSpeed;

 While TRUE Do
  Begin
   TabUse[SpeedCourant] := TRUE;
   Com.Clear;
   Com.SetBaud(TabSpeed[SpeedCourant].BD);
   { Demande status du terminal }
   Com.Write(#$0#$0#$0#$0+MPRO1+#$70#$0#$0#$0#$0);
   Com.WaitOutEmpty;

   { attendre le status de module receveur prise }

   x := _CalcC18Delay(4,SpeedCourant,Slow);

   If WaitInStr( MPRO2+#$71#$00, x, x*4, Res) Then
    Begin
     If WantSpeed=SpeedCourant Then
      Begin
       MinSpeed := SpeedCourant;
       MinSlow  := Slow;
       MinDetect := TRUE;
       Exit;
      End;

     If LastSpeed=SPEED_NIL Then LastSpeed := SpeedCourant;

     { change la vitesse }

     Com.Write(#0#0#0#0+MPRO2+Char($6B)+TabSpeed[WantSpeed].CSpeed);
     Com.WaitOutEmpty;
     Delay(200);

     SpeedCourant := WantSpeed;
    End
   Else
    If LastSpeed<>SPEED_NIL Then
     If Not(AutoSeek) Or (WantSpeed=Speed1200) Then
      Begin
       MinDetect := FALSE;
       Exit;
      End
     Else { recherche automatique }
      Begin
       If WantSpeed>Speed1200 Then Dec(WantSpeed)
       Else Inc(WantSpeed);
       SpeedCourant := LastSpeed;
      End
    Else
     Begin
      While TabUse[OrderRank[SpeedRank]] Do
       Begin
        Inc(SpeedRank);
        If SpeedRank>=NbSpeed Then
         Begin
          MinDetect := FALSE;
          Exit;
         End;
       End;
      SpeedCourant := OrderRank[SpeedRank];
     End;
  End;
End;


Function MinStatusTerm( Var St : Byte ) : Boolean;
Var Res : String;
    x   : Word;
Begin
 { Demande status du terminal }
 Com.Write(MPRO1+#$70);
 Com.WaitOutEmpty;

 { attendre le status de module receveur prise }

 x := CalcC18Delay(4);

 If WaitInStr( MPRO2+#$71#$00, x, x*4, Res ) Then
  Begin
   St := Byte(Res[4]);
   MinStatusTerm := TRUE;
  End
 Else MinStatusTerm := FALSE;
End;


Procedure MinRestoreSpeed;
Begin
 If LastSpeed<>SPEED_NIL Then
  Begin
   Com.Write(#0#0#0#0+MPRO2+Char($6B)+TabSpeed[LastSpeed].CSpeed);
   Com.WaitOutEmpty;
   Delay(200);
  End;
End;
